(in-package :cl-user)


;;; This file tests the :ALL-PATHS form


(defun ap-setup ()

  (acom-reset)

  (silently
   
   (tell '((:TAXONOMY  (Things
			(Physical-objects
			 (People  Father Child1 Child2 Child3)
			 (Genders Male Female)
			 (Heights Short Tall)
			 (Dispositions Grumpy Happy)
			 )))))

   (tell '((:SLOT  gender      (People Genders))
	   (:SLOT  child       (People People))
	   (:SLOT  height      (People Heights))
	   (:SLOT  disposition (People Dispositions))
	   ))


   )
  )


(format t "~2%run (ap-test) to test the :ALL-PATHS command.~%")

(defun ap-test (&OPTIONAL (trace-level 1))

  (ap-test-1 trace-level)
  (ap-test-2 trace-level)
  (ap-test-3 trace-level)
  (ap-test-4 trace-level)
  (ap-test-5 trace-level)
  (ap-test-6 trace-level)
  )


(defun ap-test-1 (&OPTIONAL (trace-level 1))

  (format t "~2%***** (ap-test-1): All paths in path1 succeed, All paths in path2 succeed.")
  (format t "~2%AP-TEST-1: Should succeed with two bindings: Child1 and Child2.")

  (ap-setup)
  (tell '((child  Father Child1)
	  (child  Father Child2)
	  (gender Child1 Male)
	  (gender Child2 Male)
	  (height Child1 Tall)
	  (height Child2 Tall)
	  ))
	
  (algy-trace trace-level)

  (ask '((:ALL-PATHS ((child Father ?c)) ((gender ?c Male)))))

  (algy-trace 1)
  )



(defun ap-test-2 (&OPTIONAL (trace-level 1))

  (format t "~2%***** (ap-test-2): All paths in path1 succeed, One path in path2 succeeds.")
  (format t "~2%AP-TEST-2: Should fail because second path fails for one binding.")

  (ap-setup)
  (tell '((child  Father  Child1)
	  (child  Father  Child2)
	  (gender Child1  Male)
	  (gender Child2  Female)
	  (height Child1  Tall)
	  (height Child2  Tall)
	  ))
	
  (algy-trace trace-level)

  (ask '((:ALL-PATHS ((child Father ?c)) ((gender ?c Male)))))

  (algy-trace 1)
  )



(defun ap-test-3 (&OPTIONAL (trace-level 1))

  (format t "~2%***** (ap-test-3): One path in path1 succeeds, All paths in path2 succeed.")
  (format t "~2%AP-TEST-3: Should succeed with one binding: Child1.")

  (ap-setup)
  (tell '((child  Father  Child1)
	  (child  Father  Child2)
	  (gender Child1  Male)
	  (gender Child2  Male)
	  (height Child1  Tall)
	  (height Child2  Short)
	  (disposition Child1  Happy)
	  (disposition Child2  Happy)
	  ))

  (algy-trace trace-level)
	
  (ask '((:ALL-PATHS ((child Father ?c) (height ?c Tall))
		     ((gender ?c Male)  (disposition ?c Happy)))))

  (algy-trace 1)
  )



(defun ap-test-4 (&OPTIONAL (trace-level 1))

  (format t "~2%***** (ap-test-4): Two of Three paths in path1 succeed, one path in path2 succeeds.")
  (format t "~2%AP-TEST-4: Should fail because one binding in path2 fails.")

  (ap-setup)
  (tell '((child  Father  Child1)
	  (child  Father  Child2)
	  (child  Father  Child3)
	  (gender Child1  Male)
	  (gender Child2  Male)
	  (gender Child3  Male)
	  (height Child1  Tall)
	  (height Child2  Short)
	  (height Child3  Tall)
	  (disposition Child1  Happy)
	  (disposition Child2  Grumpy)
	  ))

  (algy-trace trace-level)
	
  (ask '((:ALL-PATHS ((child Father ?c) (height ?c Tall))
		     ((gender ?c Male)  (disposition ?c Happy)))))

  (algy-trace 1)
  )



(defun ap-test-5 (&OPTIONAL (trace-level 1))

  (format t "~2%***** (ap-test-5): No paths in path1 succeed, All paths in path2 succeed.")
  (format t "~%        Note: Path2 can't use variables from path1.")
  (format t "~2%AP-TEST-5: Should succeed, but not execute the second path.")

  (ap-setup)
  (tell '((child  Father  Child1)
	  (child  Father  Child2)
	  (gender Child1  Female)
	  (gender Child2  Female)
	  (height Child1  Short)
	  (height Child2  Short)
	  ))

  (algy-trace trace-level)

  (ask '((:ALL-PATHS ((child Father ?c) (height ?c Tall))
		     ((:eval (format *standard-output* "~%ALGY-ERROR: second path executed."))
		      (member People Child1)  (member People Child2)))))

  (algy-trace 1)
  )



(defun ap-test-6 (&OPTIONAL (trace-level 1))

  (format t "~2%***** (ap-test-6): No paths in path1 succeed, No paths in path2 succeed.")
  (format t "~%        Note: Path2 can't use variables from path1.")
  (format t "~2%AP-TEST-6: Should succeed, but not execute the second path.")

  (ap-setup)
  (tell '((child  Father  Child1)
	  (child  Father  Child2)
	  (gender Child1  Male)
	  (gender Child2  Female)
	  (height Child1  Tall)
	  (height Child2  Tall)
	))

  (algy-trace trace-level)

  (ask '((:ALL-PATHS ((child Father ?c)        (height ?c Short))
		     ((:eval (format *standard-output* "~%ALGY-ERROR: second path executed."))
		      (member Heights Child1)  (member Heights Child2)))))

  (algy-trace 1)
  )

  
		       
