(in-package :cl-user)


;;; This file tests the :OR form


(defun or-setup ()

  (acom-reset)

  (silently
   
   (tell '((:TAXONOMY  (Things
			(Physical-objects
			 (People  Father Mother Child1 Child2 Child3)
			 (Genders Male Female)
			 (Heights Short Tall)
			 (Dispositions Grumpy Happy)
			 )))))

   (tell '((:SLOT  gender      (People Genders))
	   (:SLOT  child       (People People))
	   (:SLOT  height      (People Heights))
	   (:SLOT  disposition (People Dispositions))
	   ))


   )
  )


(format t "~2%(or-test-1): All paths succeed, the first result is returned.")

(defun or-test-1 (&OPTIONAL (trace-level 1))

  (format t "~2%OR-TEST-1: Should succeed with result of first path, binding = Child1.")

  (or-setup)
  (tell '((child  Father Child1)
	  (child  Father Child2)
	  (child  Father Child3)
	  (child  Mother Child2)
	  (gender Child1 Male)
	  (height Child2 Tall)
	  (disposition Child3 Happy)
	  ))
	
  (algy-trace trace-level)

  (print (ask '(
;;		(child Mother ?x)
		(child Father ?c)
		(:OR ((gender ?c Male))
		 ((height ?c Tall))
		 ((disposition ?c Happy)))
		) :collect '?c))

  (algy-trace 1)
  )


(format t "~2%(or-test-2): All paths in path1 succeed, One path in path2 succeeds.")

(defun or-test-2 (&OPTIONAL (trace-level 1))

  (format t "~2%OR-TEST-2: Should succeed on second :OR path, binding = Child2.")

  (or-setup)
  (tell '((child  Father Child1)
	  (child  Father Child2)
	  (child  Father Child3)
	  (gender Child1 Male)
	  (height Child2 Tall)
	  (disposition Child3 Happy)
	  ))
	
  (algy-trace trace-level)

  (print (ask '((child Father ?c)
		(:OR ((gender ?c Female))
		 ((height ?c Tall))
		 ((disposition ?c Happy)))

		) :collect '?c))

  (algy-trace 1)
  )


(format t "~2%(ap-test-3): One path in path1 succeeds, All paths in path2 succeed.")

(defun ap-test-3 (&OPTIONAL (trace-level 1))

  (format t "~2%AP-TEST-3: Should succeed with one binding: Child1.")

  (or-setup)
  (tell '((child  Father  Child1)
	  (child  Father  Child2)
	  (gender Child1  Male)
	  (gender Child2  Male)
	  (height Child1  Tall)
	  (height Child2  Short)
	  (disposition Child1  Happy)
	  (disposition Child2  Happy)
	  ))

  (algy-trace trace-level)
	
  (ask '((:ALL-PATHS ((child Father ?c) (height ?c Tall))
		     ((gender ?c Male)  (disposition ?c Happy)))))

  (algy-trace 1)
  )


(format t "~2%(ap-test-4): Two of Three paths in path1 succeed, one path in path2 succeeds.")

(defun ap-test-4 (&OPTIONAL (trace-level 1))

  (format t "~2%AP-TEST-4: Should fail because one binding in path2 fails.")

  (or-setup)
  (tell '((child  Father  Child1)
	  (child  Father  Child2)
	  (child  Father  Child3)
	  (gender Child1  Male)
	  (gender Child2  Male)
	  (gender Child3  Male)
	  (height Child1  Tall)
	  (height Child2  Short)
	  (height Child3  Tall)
	  (disposition Child1  Happy)
	  (disposition Child2  Grumpy)
	  ))

  (algy-trace trace-level)
	
  (ask '((:ALL-PATHS ((child Father ?c) (height ?c Tall))
		     ((gender ?c Male)  (disposition ?c Happy)))))

  (algy-trace 1)
  )


(format t "~2%(ap-test-5): No paths in path1 succeed, All paths in path2 succeed.")
(format t "~%        Note: Path2 can't use variables from path1.")

(defun ap-test-5 (&OPTIONAL (trace-level 1))

  (format t "~2%AP-TEST-5: Should succeed even though the first path failed.")

  (or-setup)
  (tell '((child  Father  Child1)
	  (child  Father  Child2)
	  (gender Child1  Female)
	  (gender Child2  Female)
	  (height Child1  Short)
	  (height Child2  Short)
	  ))

  (algy-trace trace-level)

  (ask '((:ALL-PATHS ((child Father ?c) (height ?c Tall))
		     ((member People Child1)  (member People Child2)))))

  (algy-trace 1)
  )


(format t "~2%(ap-test-6): No paths in path1 succeed, No paths in path2 succeed.")
(format t "~%        Note: Path2 can't use variables from path1.")

(defun ap-test-6 (&OPTIONAL (trace-level 1))

  (format t "~2%AP-TEST-6: Should fail because second path fails.")

  (or-setup)
  (tell '((child  Father  Child1)
	  (child  Father  Child2)
	  (gender Child1  Male)
	  (gender Child2  Female)
	  (height Child1  Tall)
	  (height Child2  Tall)
	))

  (algy-trace trace-level)

  (ask '((:ALL-PATHS ((child Father ?c)        (height ?c Short))
		     ((member Heights Child1)  (member Heights Child2)))))

  (algy-trace 1)
  )

  
		       
